@@ -1,15 +1,16 @@
 # ----------------------------------------------------------------------------
 # pyglet
 # Copyright (c) 2006-2008 Alex Holkner
+# Copyright (c) 2008-2019 pyglet contributors
 # All rights reserved.
-# 
+#
 # Redistribution and use in source and binary forms, with or without
-# modification, are permitted provided that the following conditions 
+# modification, are permitted provided that the following conditions
 # are met:
 #
 #  * Redistributions of source code must retain the above copyright
 #    notice, this list of conditions and the following disclaimer.
-#  * Redistributions in binary form must reproduce the above copyright 
+#  * Redistributions in binary form must reproduce the above copyright
 #    notice, this list of conditions and the following disclaimer in
 #    the documentation and/or other materials provided with the
 #    distribution.
@@ -32,12 +33,12 @@
 # POSSIBILITY OF SUCH DAMAGE.
 # ----------------------------------------------------------------------------
 
-'''Abstract classes used by pyglet.font implementations.
+"""Abstract classes used by pyglet.font implementations.
 
 These classes should not be constructed directly.  Instead, use the functions
 in `pyglet.font` to obtain platform-specific instances.  You can use these
 classes as a documented interface to the concrete classes.
-'''
+"""
 from builtins import chr
 from builtins import str
 from builtins import map
@@ -52,15 +53,12 @@ import unicodedata
 from pyglet.gl import *
 from pyglet import image
 
-_other_grapheme_extend = \
-    list(map(chr, [0x09be, 0x09d7, 0x0be3, 0x0b57, 0x0bbe, 0x0bd7, 0x0cc2,
-                 0x0cd5, 0x0cd6, 0x0d3e, 0x0d57, 0x0dcf, 0x0ddf, 0x200c,
-                 0x200d, 0xff9e, 0xff9f])) # skip codepoints above U+10000
-_logical_order_exception = \
-    list(map(chr, list(range(0xe40, 0xe45)) + list(range(0xec0, 0xec4))))
+_other_grapheme_extend = list(map(chr, [0x09be, 0x09d7, 0x0be3, 0x0b57, 0x0bbe, 0x0bd7, 0x0cc2,
+                                        0x0cd5, 0x0cd6, 0x0d3e, 0x0d57, 0x0dcf, 0x0ddf, 0x200c,
+                                        0x200d, 0xff9e, 0xff9f])) # skip codepoints above U+10000
+_logical_order_exception = list(map(chr, list(range(0xe40, 0xe45)) + list(range(0xec0, 0xec4))))
 
-_grapheme_extend = lambda c, cc: \
-    cc in ('Me', 'Mn') or c in _other_grapheme_extend
+_grapheme_extend = lambda c, cc: cc in ('Me', 'Mn') or c in _other_grapheme_extend
 
 _CR = u'\u000d'
 _LF = u'\u000a'
@@ -71,6 +69,7 @@ _extend = lambda c, cc: _grapheme_extend(c, cc) or \
 _prepend = lambda c, cc: c in _logical_order_exception
 _spacing_mark = lambda c, cc: cc == 'Mc' and c not in _other_grapheme_extend
 
+
 def _grapheme_break(left, right):
     # GB1
     if left is None:
@@ -111,8 +110,9 @@ def _grapheme_break(left, right):
     # GB10
     return True
 
+
 def get_grapheme_clusters(text):
-    '''Implements Table 2 of UAX #29: Grapheme Cluster Boundaries.
+    """Implements Table 2 of UAX #29: Grapheme Cluster Boundaries.
 
     Does not currently implement Hangul syllable rules.
     
@@ -124,7 +124,7 @@ def get_grapheme_clusters(text):
 
     :rtype: List of `unicode`
     :return: List of Unicode grapheme clusters
-    '''
+    """
     clusters = []
     cluster = ''
     left = None
@@ -143,8 +143,9 @@ def get_grapheme_clusters(text):
         clusters.append(cluster)
     return clusters
 
+
 class Glyph(image.TextureRegion):
-    '''A single glyph located within a larger texture.
+    """A single glyph located within a larger texture.
 
     Glyphs are drawn most efficiently using the higher level APIs, for example
     `GlyphString`.
@@ -156,13 +157,13 @@ class Glyph(image.TextureRegion):
             The vertices of this glyph, with (0,0) originating at the
             left-side bearing at the baseline.
 
-    '''
+    """
 
     advance = 0
     vertices = (0, 0, 0, 0)
 
     def set_bearings(self, baseline, left_side_bearing, advance):
-        '''Set metrics for this glyph.
+        """Set metrics for this glyph.
 
         :Parameters:
             `baseline` : int
@@ -173,7 +174,7 @@ class Glyph(image.TextureRegion):
             `advance` : int
                 Distance to move the horizontal advance to the next glyph.
 
-        '''
+        """
         self.advance = advance
         self.vertices = (
             left_side_bearing,
@@ -182,20 +183,20 @@ class Glyph(image.TextureRegion):
             -baseline + self.height)
 
     def draw(self):
-        '''Debug method.
+        """Debug method.
         
         Use the higher level APIs for performance and kerning.
-        '''
+        """
         glBindTexture(GL_TEXTURE_2D, self.owner.id)
         glBegin(GL_QUADS)
         self.draw_quad_vertices()
         glEnd()
 
     def draw_quad_vertices(self):
-        '''Debug method. 
+        """Debug method. 
 
         Use the higher level APIs for performance and kerning.
-        '''
+        """
         glTexCoord3f(*self.tex_coords[:3])
         glVertex2f(self.vertices[0], self.vertices[1])
         glTexCoord3f(*self.tex_coords[3:6])
@@ -206,26 +207,27 @@ class Glyph(image.TextureRegion):
         glVertex2f(self.vertices[0], self.vertices[3])
 
     def get_kerning_pair(self, right_glyph):
-        '''Not implemented.
-        '''
+        """Not implemented.
+        """
         return 0
 
+
 class GlyphTextureAtlas(image.Texture):
-    '''A texture within which glyphs can be drawn.
-    '''
+    """A texture within which glyphs can be drawn.
+    """
     region_class = Glyph
     x = 0
     y = 0
     line_height = 0
 
     def apply_blend_state(self):
-        '''Set the OpenGL blend state for the glyphs in this texture.
-        '''
+        """Set the OpenGL blend state for the glyphs in this texture.
+        """
         glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
         glEnable(GL_BLEND)
 
     def fit(self, image):
-        '''Place `image` within this texture.
+        """Place `image` within this texture.
 
         :Parameters:
             `image` : `pyglet.image.AbstractImage`
@@ -234,7 +236,7 @@ class GlyphTextureAtlas(image.Texture):
         :rtype: `Glyph`
         :return: The glyph representing the image from this texture, or None
             if the image doesn't fit.
-        '''
+        """
         if image.width > self.width or image.height > self.height:
             return None
 
@@ -253,22 +255,25 @@ class GlyphTextureAtlas(image.Texture):
             self.x += image.width + 1
         return region
 
+
 class GlyphRenderer(object):
-    '''Abstract class for creating glyph images.
-    '''
+    """Abstract class for creating glyph images.
+    """
     def __init__(self, font):
         pass
 
     def render(self, text):
         raise NotImplementedError('Subclass must override')
 
+
 class FontException(Exception):
-    '''Generic exception related to errors from the font module.  Typically
-    these relate to invalid font data.'''
+    """Generic exception related to errors from the font module.  Typically
+    these relate to invalid font data."""
     pass
 
+
 class Font(object):
-    '''Abstract font class able to produce glyphs.
+    """Abstract font class able to produce glyphs.
 
     To construct a font, use :py:func:`pyglet.font.load`, which will instantiate the
     platform-specific font class.
@@ -281,10 +286,12 @@ class Font(object):
             Maximum ascent above the baseline, in pixels.
         `descent` : int
             Maximum descent below the baseline, in pixels. Usually negative.
-    '''
+    """
     texture_width = 256
     texture_height = 256
     texture_internalformat = GL_ALPHA
+    texture_min_filter = GL_LINEAR
+    texture_mag_filter = GL_LINEAR
 
     # These should also be set by subclass when known
     ascent = 0
@@ -299,7 +306,7 @@ class Font(object):
 
     @classmethod
     def add_font_data(cls, data):
-        '''Add font data to the font loader.
+        """Add font data to the font loader.
 
         This is a class method and affects all fonts loaded.  Data must be
         some byte string of data, for example, the contents of a TrueType font
@@ -308,23 +315,23 @@ class Font(object):
 
         There is no way to instantiate a font given the data directly, you
         must use :py:func:`pyglet.font.load` specifying the font name.
-        '''
+        """
         pass
 
     @classmethod
     def have_font(cls, name):
-        '''Determine if a font with the given name is installed.
+        """Determine if a font with the given name is installed.
 
         :Parameters:
             `name` : str
                 Name of a font to search for
 
         :rtype: bool
-        '''
+        """
         return True
 
     def create_glyph(self, image):
-        '''Create a glyph using the given image.
+        """Create a glyph using the given image.
 
         This is used internally by `Font` subclasses to add glyph data
         to the font.  Glyphs are packed within large textures maintained by
@@ -339,7 +346,7 @@ class Font(object):
                 The image to write to the font texture.
 
         :rtype: `Glyph`
-        '''
+        """
         glyph = None
         self._adapt_texture_size(image)
         for texture in self.textures:
@@ -348,20 +355,22 @@ class Font(object):
                 break
         if not glyph:
             texture = self.texture_class.create_for_size(GL_TEXTURE_2D,
-                self.texture_width, self.texture_height,
-                self.texture_internalformat)
+                                                         self.texture_width,
+                                                         self.texture_height,
+                                                         self.texture_internalformat,
+                                                         self.texture_min_filter,
+                                                         self.texture_mag_filter)
             self.textures.insert(0, texture)
             glyph = texture.fit(image)
         return glyph
 
     def _adapt_texture_size(self, image):
-        if image.width > self.texture_width or \
-           image.height > self.texture_height:
+        if image.width > self.texture_width or image.height > self.texture_height:
             largest_dimension = max(image.width, image.height)
             self.texture_height = self.texture_width = largest_dimension * 4
 
     def get_glyphs(self, text):
-        '''Create and return a list of Glyphs for `text`.
+        """Create and return a list of Glyphs for `text`.
 
         If any characters do not have a known glyph representation in this
         font, a substitution will be made.
@@ -371,7 +380,7 @@ class Font(object):
                 Text to render.
 
         :rtype: list of `Glyph`
-        '''
+        """
         glyph_renderer = None
         glyphs = []         # glyphs that are committed.
         for c in get_grapheme_clusters(str(text)):
@@ -388,7 +397,7 @@ class Font(object):
 
 
     def get_glyphs_for_width(self, text, width):
-        '''Return a list of glyphs for `text` that fit within the given width.
+        """Return a list of glyphs for `text` that fit within the given width.
         
         If the entire text is larger than 'width', as much as possible will be
         used while breaking after a space or zero-width space character.  If a
@@ -411,7 +420,7 @@ class Font(object):
         :rtype: list of `Glyph`
 
         :see: `GlyphString`
-        '''
+        """
         glyph_renderer = None
         glyph_buffer = []   # next glyphs to be added, as soon as a BP is found
         glyphs = []         # glyphs that are committed.
@@ -445,5 +454,3 @@ class Font(object):
             glyphs = glyph_buffer
 
         return glyphs
-
-
