@@ -150,6 +150,7 @@ from pyglet.compat import asbytes, bytes_type, BytesIO
 from .codecs import ImageEncodeException, ImageDecodeException
 from .codecs import add_default_image_codecs, add_decoders, add_encoders
 from .codecs import get_animation_decoders, get_decoders, get_encoders
+from .animation import Animation, AnimationFrame
 from . import atlas
 
 
@@ -210,6 +211,46 @@ def load(filename, file=None, decoder=None):
             opened_file.close()
 
 
+def load_animation(filename, file=None, decoder=None):
+    """Load an animation from a file.
+
+    Currently, the only supported format is GIF.
+
+    :Parameters:
+        `filename` : str
+            Used to guess the animation format, and to load the file if `file`
+            is unspecified.
+        `file` : file-like object or None
+            File object containing the animation stream.
+        `decoder` : ImageDecoder or None
+            If unspecified, all decoders that are registered for the filename
+            extension are tried.  If none succeed, the exception from the
+            first decoder is raised.
+
+    :rtype: Animation
+    """
+    if not file:
+        file = open(filename, 'rb')
+    if not hasattr(file, 'seek'):
+        file = BytesIO(file.read())
+
+    if decoder:
+        return decoder.decode_animation(file, filename)
+    else:
+        first_exception = None
+        for decoder in get_animation_decoders(filename):
+            try:
+                image = decoder.decode_animation(file, filename)
+                return image
+            except ImageDecodeException as e:
+                first_exception = first_exception or e
+                file.seek(0)
+
+        if not first_exception:
+            raise ImageDecodeException('No image decoders are available')
+        raise first_exception
+
+
 def create(width, height, pattern=None):
     """Create an image optionally filled with the given pattern.
 
@@ -233,7 +274,14 @@ def create(width, height, pattern=None):
     return pattern.create_image(width, height)
 
 
-def color_as_bytes(color):
+def get_max_texture_size():
+    """Query the maximum texture size available"""
+    size = c_int()
+    glGetIntegerv(GL_MAX_TEXTURE_SIZE, size)
+    return size.value
+
+
+def _color_as_bytes(color):
     if sys.version.startswith('2'):
         return '%c%c%c%c' % color
     else:
@@ -242,13 +290,6 @@ def color_as_bytes(color):
         return bytes(color)
 
 
-def get_max_texture_size():
-    """Query the maximum texture size available"""
-    size = c_int()
-    glGetIntegerv(GL_MAX_TEXTURE_SIZE, size)
-    return size.value
-
-
 def _nearest_pow2(v):
     # From http://graphics.stanford.edu/~seander/bithacks.html#RoundUpPowerOf2
     # Credit: Sean Anderson
@@ -295,7 +336,7 @@ class SolidColorImagePattern(ImagePattern):
                 color to fill with.
 
         """
-        self.color = color_as_bytes(color)
+        self.color = _color_as_bytes(color)
 
     def create_image(self, width, height):
         data = self.color * width * height
@@ -320,8 +361,8 @@ class CheckerImagePattern(ImagePattern):
                 bottom-left corners of the image.
 
         """
-        self.color1 = color_as_bytes(color1)
-        self.color2 = color_as_bytes(color2)
+        self.color1 = _color_as_bytes(color1)
+        self.color2 = _color_as_bytes(color2)
 
     def create_image(self, width, height):
         hw = width // 2
@@ -1167,8 +1208,7 @@ class CompressedImageData(AbstractImage):
     _current_texture = None
     _current_mipmap_texture = None
 
-    def __init__(self, width, height, gl_format, data,
-                 extension=None, decoder=None):
+    def __init__(self, width, height, gl_format, data, extension=None, decoder=None):
         """Construct a CompressedImageData with the given compressed data.
 
         :Parameters:
@@ -1189,9 +1229,6 @@ class CompressedImageData(AbstractImage):
                 required extension is not present.
                 
         """
-        if not _is_pow2(width) or not _is_pow2(height):
-            raise ImageException('Dimensions of %r must be powers of 2' % self)
-
         super(CompressedImageData, self).__init__(width, height)
         self.data = data
         self.gl_format = gl_format
@@ -1235,27 +1272,27 @@ class CompressedImageData(AbstractImage):
 
     def get_texture(self, rectangle=False, force_rectangle=False):
         if force_rectangle:
-            raise ImageException(
-                'Compressed texture rectangles not supported')
+            raise ImageException('Compressed texture rectangles not supported')
 
         if self._current_texture:
             return self._current_texture
 
-        texture = Texture.create_for_size(
-            GL_TEXTURE_2D, self.width, self.height)
+        tex_id = GLuint()
+        glGenTextures(1, byref(tex_id))
+        texture = Texture(self.width, self.height, GL_TEXTURE_2D, tex_id.value)
+        glBindTexture(GL_TEXTURE_2D, tex_id)
+        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, Texture.default_min_filter)
+        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, Texture.default_mag_filter)
+
         if self.anchor_x or self.anchor_y:
             texture.anchor_x = self.anchor_x
             texture.anchor_y = self.anchor_y
 
-        glBindTexture(texture.target, texture.id)
-        glTexParameteri(texture.target, GL_TEXTURE_MIN_FILTER, texture.min_filter)
-        glTexParameteri(texture.target, GL_TEXTURE_MAG_FILTER, texture.mag_filter)
-
         if self._have_extension():
-            glCompressedTexImage2DARB(texture.target, texture.level,
-                                      self.gl_format,
-                                      self.width, self.height, 0,
-                                      len(self.data), self.data)
+            glCompressedTexImage2D(texture.target, texture.level,
+                                   self.gl_format,
+                                   self.width, self.height, 0,
+                                   len(self.data), self.data)
         else:
             image = self.decoder(self.data, self.width, self.height)
             texture = image.get_texture()
@@ -1275,8 +1312,7 @@ class CompressedImageData(AbstractImage):
             # just return a non-mipmapped texture.
             return self.get_texture()
 
-        texture = Texture.create_for_size(
-            GL_TEXTURE_2D, self.width, self.height)
+        texture = Texture.create_for_size(GL_TEXTURE_2D, self.width, self.height)
         if self.anchor_x or self.anchor_y:
             texture.anchor_x = self.anchor_x
             texture.anchor_y = self.anchor_y
@@ -2320,186 +2356,5 @@ class TextureGrid(TextureRegion, UniformTextureSequence):
         return iter(self.items)
 
 
-# --------------------------------------------------------------------------
-# Animation stuff here.  Vote on if this should be in pyglet.image.animation
-# or just leave it tacked on here.
-
-# TODO: 
-#       conversion Animation ->  media.Source
-#       move to another module?  
-#          pyglet.animation?
-#          pyglet.image.animation?
-
-def load_animation(filename, file=None, decoder=None):
-    """Load an animation from a file.
-
-    Currently, the only supported format is GIF.
-
-    :Parameters:
-        `filename` : str
-            Used to guess the animation format, and to load the file if `file`
-            is unspecified.
-        `file` : file-like object or None
-            File object containing the animation stream.
-        `decoder` : ImageDecoder or None
-            If unspecified, all decoders that are registered for the filename
-            extension are tried.  If none succeed, the exception from the
-            first decoder is raised.
-
-    :rtype: Animation
-    """
-    if not file:
-        file = open(filename, 'rb')
-    if not hasattr(file, 'seek'):
-        file = BytesIO(file.read())
-
-    if decoder:
-        return decoder.decode_animation(file, filename)
-    else:
-        first_exception = None
-        for decoder in get_animation_decoders(filename):
-            try:
-                image = decoder.decode_animation(file, filename)
-                return image
-            except ImageDecodeException as e:
-                first_exception = first_exception or e
-                file.seek(0)
-
-        if not first_exception:
-            raise ImageDecodeException('No image decoders are available')
-        raise first_exception
-
-
-class Animation(object):
-    """Sequence of images with timing information.
-
-    If no frames of the animation have a duration of ``None``, the animation
-    loops continuously; otherwise the animation stops at the first frame with
-    duration of ``None``.
-
-    :Ivariables:
-        `frames` : list of `~pyglet.image.AnimationFrame`
-            The frames that make up the animation.
-
-    """
-    def __init__(self, frames):
-        """Create an animation directly from a list of frames.
-
-        :Parameters:
-            `frames` : list of `~pyglet.image.AnimationFrame`
-                The frames that make up the animation.
-
-        """
-        assert len(frames)
-        self.frames = frames
-
-    def add_to_texture_bin(self, bin):
-        """Add the images of the animation to a :py:class:`~pyglet.image.atlas.TextureBin`.
-
-        The animation frames are modified in-place to refer to the texture bin
-        regions.
-
-        :Parameters:
-            `bin` : `~pyglet.image.atlas.TextureBin`
-                Texture bin to upload animation frames into.
-
-        """
-        for frame in self.frames:
-            frame.image = bin.add(frame.image)
-
-    def get_transform(self, flip_x=False, flip_y=False, rotate=0):
-        """Create a copy of this animation applying a simple transformation.
-
-        The transformation is applied around the image's anchor point of
-        each frame.  The texture data is shared between the original animation
-        and the transformed animation.
-
-        :Parameters:
-            `flip_x` : bool
-                If True, the returned animation will be flipped horizontally.
-            `flip_y` : bool
-                If True, the returned animation will be flipped vertically.
-            `rotate` : int
-                Degrees of clockwise rotation of the returned animation.  Only
-                90-degree increments are supported.
-
-        :rtype: :py:class:`~pyglet.image.Animation`
-        """
-        frames = [AnimationFrame(frame.image.get_texture().get_transform(flip_x, flip_y, rotate),
-                                 frame.duration) for frame in self.frames]
-        return Animation(frames)
-
-    def get_duration(self):
-        """Get the total duration of the animation in seconds.
-
-        :rtype: float
-        """
-        return sum([frame.duration for frame in self.frames if frame.duration is not None])
-
-    def get_max_width(self):
-        """Get the maximum image frame width.
-
-        This method is useful for determining texture space requirements: due
-        to the use of ``anchor_x`` the actual required playback area may be
-        larger.
-
-        :rtype: int
-        """
-        return max([frame.image.width for frame in self.frames])
-
-    def get_max_height(self):
-        """Get the maximum image frame height.
-
-        This method is useful for determining texture space requirements: due
-        to the use of ``anchor_y`` the actual required playback area may be
-        larger.
-
-        :rtype: int
-        """
-        return max([frame.image.height for frame in self.frames])
-
-    @classmethod
-    def from_image_sequence(cls, sequence, period, loop=True):
-        """Create an animation from a list of images and a constant framerate.
-
-        :Parameters:
-            `sequence` : list of `~pyglet.image.AbstractImage`
-                Images that make up the animation, in sequence.
-            `period` : float
-                Number of seconds to display each image.
-            `loop` : bool
-                If True, the animation will loop continuously.
-
-        :rtype: :py:class:`~pyglet.image.Animation`
-        """
-        frames = [AnimationFrame(image, period) for image in sequence]
-        if not loop:
-            frames[-1].duration = None
-        return cls(frames)
-
-
-class AnimationFrame(object):
-    """A single frame of an animation."""
-
-    __slots__ = 'image', 'duration'
-
-    def __init__(self, image, duration):
-        """Create an animation frame from an image.
-
-        :Parameters:
-            `image` : `~pyglet.image.AbstractImage`
-                The image of this frame.
-            `duration` : float
-                Number of seconds to display the frame, or ``None`` if it is
-                the last frame in the animation.
-
-        """
-        self.image = image
-        self.duration = duration
-
-    def __repr__(self):
-        return 'AnimationFrame(%r, %r)' % (self.image, self.duration)
-
-
 # Initialise default codecs
 add_default_image_codecs()
